﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.Runtime.InteropServices;   ///< To use Toradex C Win32 DLLs.

namespace vcsharp_sevensegment_demo
{
    public partial class Form1 : Form
    {
        [DllImport("GPIOLib.dll", CharSet = CharSet.Auto)]                              ///< Importing DLL.
        public static extern bool SetPinAltFn(int pinNum, int altFn, bool dirOut);      ///< External function declaration.

        [DllImport("GPIOLib.dll", CharSet = CharSet.Auto)]
        public static extern void InitGPIOLib();

        [DllImport("GPIOLib.dll", CharSet = CharSet.Auto)]
        public static extern bool SetPinLevel(int gpioNum, int val);

        [DllImport("GPIOLib.dll", CharSet = CharSet.Auto)]
        public static extern bool GetPinLevel(int gpioNum);

        [DllImport("GPIOLib.dll", CharSet = CharSet.Auto)]
        public static extern void DeInitGPIOLib();

        private const int PIN_A = 92;          ///< Iris connector X16 pin 11, used as GPIO.
        private const int PIN_B = 98;          ///< Iris connector X16 pin 13, used as GPIO.
        private const int PIN_C = 103;         ///< Iris connector X16 pin 15, used as GPIO.
        private const int PIN_D = 101;         ///< Iris connector X16 pin 16, used as GPIO.
        private const int PIN_E = 97;          ///< Iris connector X16 pin 17, used as GPIO.
        private const int PIN_F = 85;          ///< Iris connector X16 pin 18, used as GPIO.
        private const int PIN_G = 79;          ///< Iris connector X16 pin 19, used as GPIO.
        private const int PIN_DOT = 45;        ///< Iris connector X16 pin 20, used as GPIO.
        private const int SELECTION_SEGMENT_0 = 90;     ///< Iris connector X16 pin 10, used as GPIO.
        private const int SELECTION_SEGMENT_1 = 28;     ///< Iris connector X16 pin 38, used as GPIO.
        private const int SELECTION_SEGMENT_2 = 30;     ///< Iris connector X16 pin 39, used as GPIO.
        private const int SELECTION_SEGMENT_3 = 67;     ///< Iris connector X16 pin 40, used as GPIO.

        private const int GPIO = -1;
        private const int HIGH = 1;
        private const int LOW = 0;
        private const bool DIR_IN = false;
        private const bool DIR_OUT = true;
        private const int SELECTION_PIN_LOGIC = HIGH;
        private const int SELECTION_PIN_LOGIC_REVERSE = (SELECTION_PIN_LOGIC == HIGH) ? LOW : HIGH;
        private int[] selectionPin = new int[]{SELECTION_SEGMENT_0, SELECTION_SEGMENT_1, SELECTION_SEGMENT_2, SELECTION_SEGMENT_3};
        private int[] dataPin = new int[]{PIN_A, PIN_B, PIN_C, PIN_D, PIN_E, PIN_F, PIN_G, PIN_DOT};
        private int[] displayPattern = new int[]{0x3F, 0x06, 0x5B, 0x4F, 0x66, 0x6D, 0x7D, 0x07, 0x7F, 0x6F};
        private int[] digit = new int[4];
        private bool[] selectSegments = new bool[4];
        private int lengthOfData = 0;
   
        public Form1()
        {
            byte count = 0;
            InitializeComponent();
            InitGPIOLib();
            /// Configures segment data pin as output.
            for (count = 0; count < 8; count++)
            {
                SetPinAltFn(dataPin[count], GPIO, DIR_OUT);
                SetPinLevel(dataPin[count], LOW);
            }
            /// Configures selection pin as output.
            for (count = 0; count < 4; count++)
            {
                SetPinAltFn(selectionPin[count], GPIO, DIR_OUT);
                SetPinLevel(selectionPin[count], SELECTION_PIN_LOGIC_REVERSE);
            } 
        }

        /// Sends display value on data pins.
        private bool SendSegmentData(byte displayValue)
        {
            byte count = 0;
            for (count = 0; count < 8; count++)
            {
                SetPinLevel(dataPin[count], ((displayPattern[displayValue] >> count) & 0x01));
            }
            return true;
        }

        /// Displays data on seven segment.
        void SegmentDisplay()
        {
            byte selection = 0;
            for (selection = 0; selection < lengthOfData; selection++)
            {
                /// Sets all selection pins.
                SetPinLevel(SELECTION_SEGMENT_0, SELECTION_PIN_LOGIC_REVERSE);
                SetPinLevel(SELECTION_SEGMENT_1, SELECTION_PIN_LOGIC_REVERSE);
                SetPinLevel(SELECTION_SEGMENT_2, SELECTION_PIN_LOGIC_REVERSE);
                SetPinLevel(SELECTION_SEGMENT_3, SELECTION_PIN_LOGIC_REVERSE);
                if (selectSegments[selection] == false)
                {
                    continue;
                }
                SendSegmentData((byte)digit[selection]);
                /// Sets a particular selection pin to select a segment.
                SetPinLevel(selectionPin[selection], SELECTION_PIN_LOGIC);
                Thread.Sleep(5);
            }
        }

        private void displayButton_Click(object sender, EventArgs e)
        {
            int count = 0;
            int dataEntered = 0;
            if (displayButton.Text == "Display")
            {
                try
                {
                    dataEntered = Convert.ToInt32(displayData.Text);
                    lengthOfData = displayData.Text.Length;
                    displayData.Enabled = false;
                    for (count = 0; count < lengthOfData; count++)
                    {
                        digit[count] = displayData.Text[count] - '0';
                    }
                }
                catch
                {
                    MessageBox.Show("Data must be in integer format.", "Data Exception Error");
                    return;
                }
                displayButton.Text = "Stop";
                dataUpdateTimer.Enabled = true;
            }
            else
            {
                dataUpdateTimer.Enabled = false;
                displayData.Enabled = true;
                displayButton.Text = "Display";
                /// Disables all segments and data lines.
                for (count = 0; count < 8; count++)
                {
                    if (count < 4)
                    {
                        SetPinLevel(selectionPin[count], SELECTION_PIN_LOGIC_REVERSE);
                    }
                    SetPinLevel(dataPin[count], LOW);
                }
            }
        }

        private void segment1_CheckStateChanged(object sender, EventArgs e)
        {
            selectSegments[0] = segment1.Checked;
        }

        private void segment2_CheckStateChanged(object sender, EventArgs e)
        {
            selectSegments[1] = segment2.Checked;
        }

        private void segment3_CheckStateChanged(object sender, EventArgs e)
        {
            selectSegments[2] = segment3.Checked;
        }

        private void segment4_CheckStateChanged(object sender, EventArgs e)
        {
            selectSegments[3] = segment4.Checked;
        }

        private void dataUpdateTimer_Tick(object sender, EventArgs e)
        {
            /// Displays data.
            SegmentDisplay();
        }

        private void Form1_Closing(object sender, CancelEventArgs e)
        {
            if (displayButton.Text == "Stop")
            {
                displayButton_Click(sender, e);
            }
            /// De-initializes library and releases all the resources.
            DeInitGPIOLib();
        }
    }
}